<?php
// $Id$
/**
 * @file
 * Defines the achievement entity.  Used to list achievements for toons and guilds.
 */

/**
 * Called from hook_entity_info().
 */
function wowtoon_entity_wowachievement_info() {
  $return = array(
    'wowachievement' => array(
      'label' => t('WoW Achievement'),
      'controller class' => 'WoWAchievementController',
      'base table' => 'wowachievement_entity',
      'fieldable' => TRUE,
      'entity keys' => array(
        'id' => 'aid',
      ),
      'bundles' => array(
        'wowachievement' => array(
          'label' => t('WoWAchievement'),
          'admin' => array(
            'path' => 'admin/config/wowguild/TODO',
            'access arguments' => array('access content'),
          ),
        ),
      ),
      'label callback' => 'entity_class_label',
      'uri callback' => 'entity_class_uri',
      'view modes' => array(
        'full' => array(
          'label' => t('full'),
          'custom settings' => FALSE,
        ),
        'teaser' => array(
          'label' => t('teaser'),
          'custom settings' => FALSE,
        ),
        'feed' => array(
          'label' => t('feed'),
          'custom settings' => FALSE,
        ),
      ),
    ),
  );
  return $return;  
}


/**
 * Load multiple test entities based on certain conditions.
 *
 * @param $pids
 *   An array of entity IDs.
 * @param $conditions
 *   An array of conditions to match against the {entity} table.
 * @param $reset
 *   A boolean indicating that the internal cache should be reset.
 * @return
 *   An array of test entity objects, indexed by pid.
 */
function wowachievement_load_multiple($aids = array(), $conditions = array(), $reset = FALSE) {
  return entity_load('wowachievement', $aids, $conditions, $reset);
}

/**
 * hook for menu %wowtoon items
 * 
 * @param integer $tid
 * @param bool $reset - Don't use cache
 */
function wowachievement_load($aid, $reset = FALSE) {
  $toons = wowachievement_load_multiple(array($aid), array(), $reset);
  return $toons?reset($toons):FALSE;
}

/**
 * Load achievements for multiple toon->tids.
 * 
 * @param unknown_type $tids
 * @return An|multitype:
 */
function wowachievement_load_tids($tids = array()) {
  if (!empty($tids)) {
    $query = db_select('wowachievement_entity', 'a');
    $query->fields('a', array('aid'));
    $query->condition('keyid', $tids, 'IN');
    $query->condition('type', 100, '<');
    $query->condition('status', 1);
    $query->orderby('created', 'DESC');
    $aids = $query->execute()->fetchCol();
    
    $achievements = wowachievement_load_multiple($aids);
    return $achievements;
  }
  else {
    return array();
  }
}
function wowachievement_load_tid($tid = NULL) {
  if ($tid) {
    $query = db_select('wowachievement_entity', 'a');
    $query->fields('a', array('aid'));
    $query->condition('keyid', $tid);
    $query->condition('type', 100, '<');
    $query->condition('status', 1);
    $query->orderby('created', 'DESC');
    $aids = $query->execute()->fetchCol();
    
    $achievements = wowachievement_load_multiple($aids);
    return $achievements;
  }
  else {
    return array();
  }
}
function wowachievement_load_gid($gid = NULL) {
  if ($gid) {
    $query = db_select('wowachievement_entity', 'a');
    $query->fields('a', array('aid'));
    $query->condition('keyid', $gid);
    $query->condition('type', 100, '>=');
    $query->condition('status', 1);
    $query->orderby('created', 'DESC');
    $aids = $query->execute()->fetchCol();
    $achievements = wowachievement_load_multiple($aids);
    return $achievements;
  }
  else {
    return array();
  }
}

/**
 * Save wowtoon entity
 */
function wowachievement_save(&$edit) {
  field_attach_presave('wowachievement', $edit);
  
  module_invoke_all('wowachievement_presave', $edit);
  module_invoke_all('entity_presave', $edit, 'wowachievement');
  
  if (!empty($edit->aid)) {
    drupal_write_record('wowachievement_entity', $edit, 'aid');
    field_attach_update('wowachievement', $edit);
    module_invoke_all('entity_update', $edit, 'wowachievement');
  }
  else {
    drupal_write_record('wowachievement_entity', $edit);
    field_attach_insert('wowachievement', $edit);
    module_invoke_all('entity_insert', $edit, 'wowachievement');
  }
  
  return $edit;
}
/**
 * 'Delete' wowtoon.
 * if $force is TRUE, delete the entity, otherwise clear user settings and 'un owns' toon.
 */
function wowachievement_delete($aid) {
  db_delete('wowachievement_entity')
    ->condition('aid', $aid)
    ->execute();
  
  module_invoke_all('entity_delete', $aid, 'wowachievement');
}

/**
 * Main class for wowtoon entities.
 */
class WoWAchievementController extends DrupalDefaultEntityController {
  protected function attachLoad(&$achievements, $revision_id = FALSE) {
    
    foreach ($achievements as $achievement) {
      $achievements[$achievement->aid]->created_ago = format_interval(REQUEST_TIME - $achievement->created, 1) . ' ' . t('ago');
    }

    // Let fields attach themselves
    parent::attachLoad($achievements, $revision_id);
  }
  
  /**
   * Specifies the default label, which is picked up by label() by default.
   */
  protected function defaultLabel() {
    return 'unknown';
  }

  /**
   * Specifies the default uri, which is picked up by uri() by default.
   */
  protected function defaultURI() {
    return array('path' => 'toon/' . $achievement->tid);
  }
}

/**
 * Build content for specified view_mode
 * 
 * @param unknown_type $node
 * @param unknown_type $view_mode
 *   - full, teaser, popup, banner
 * @param unknown_type $langcode
 */
function wowachievement_build_content(&$achievement, $view_mode = 'full', $langcode = NULL) {
  if (!isset($langcode)) {
    $langcode = $GLOBALS['language_content']->language;
  }

  // Remove previously built content, if exists.
  $achievement->content = array();

  // Build fields content.
  // In case of a multiple view, node_view_multiple() already ran the
  // 'prepare_view' step. An internal flag prevents the operation from running
  // twice.
  field_attach_prepare_view('wowachievement', array($achievement->aid => $achievement), $view_mode);
  entity_prepare_view('wowachievement', array($achievement->aid => $achievement));
  $achievement->content += field_attach_view('wowachievement', $achievement, $view_mode, $langcode);

  if ($achievement->type < 100) {
    $achievement->toon = wowtoon_load($achievement->keyid);
  }
  else {
    $achievement->guild = wowguild_load($achievement->keyid);
  }
  
  $achievement->content['created'] = $achievement->created;
  $achievement->content['created_ago'] = $achievement->created_ago;
  $achievement->content['description'] = $achievement->description;
  $achievement->content['icon'] = $achievement->icon;
  
  if ($view_mode == 'full') {
  }
  if ($view_mode == 'teaser') {
  }
  // Allow modules to make their own additions to the node.
  module_invoke_all('wowtoon_view', $achievement, $view_mode, $langcode);
  module_invoke_all('entity_view', $achievement, 'wowtoon', $view_mode, $langcode);
}

/**
 * Display an achievement.
 * 
 * @param {wowtoon} $toon
 * @param string $view_mode
 * @param string $langcode
 */
function wowachievement_view($achievement, $view_mode = 'full', $langcode = NULL) {
  
  if (!isset($langcode)) {
    $langcode = $GLOBALS['language_content']->language;
  }
  
  // Populate $toon->content with a render() array.
  wowachievement_build_content($achievement, $view_mode, $langcode);

  $build = $achievement->content;
  // We don't need duplicate rendering info in node->content.
  unset($achievement->content);
  
  $build = array_merge($build, array(
    '#theme' => 'wowachievement',
    '#achievement' => $achievement,
    '#view_mode' => $view_mode,
    '#language' => $langcode,
  ));
  /*
  $build['user_picture'] =  array(
    '#theme' => 'wowtoon',
    '#elements' => $achievement->toon,
    '#view_mode' => 'postedby',
    '#language' => $langcode,
  );
  */

  // Allow modules to modify the structured node.
  $type = 'wowachievement';
  drupal_alter(array('wowachievement_view', 'entity_view'), $build, $type);
  
  return $build;
}  


function wowachievement_view_multiple($achievements, $view_mode = 'full', $langcode = NULL) {
  $content = array();
  foreach ($achievements as $achievement) {
    $content[] = wowachievement_view($achievement, $view_mode, $langcode);
  }
  return $content;
}


/**
 * Process variables for wowtoon.tpl.php
 * 
 * @see wowtoon_theme()
 * @see wowtoon.tpl.php
 * @see wowtoon-teaser.tpl.php
 * @see wowtoon-popup.tpl.php
 */
function template_preprocess_wowachievement(&$variables) {
  $variables['view_mode'] = $variables['elements']['#view_mode'];
  // Provide a distinct $teaser boolean.
  $variables['teaser'] = $variables['view_mode'] == 'teaser';
  $variables['achievement'] = $variables['elements']['#achievement'];
  $achievement = $variables['achievement'];

  //print("<pre>");
  //print_r($achievement);
  //die();
  
  if (isset($achievement->toon)) {
    $variables['toon'] = $achievement->toon;
    $variables['inset']     = wowtoon_get_render($achievement->toon, 'inset');
    $variables['card']      = wowtoon_get_render($achievement->toon, 'card');
    $variables['avatar']    = wowtoon_get_render($achievement->toon, 'avatar');
    
    $variables['toonlink'] = url('toon/' . $achievement->toon->tid);
    $variables['nameatrealm'] = check_plain($achievement->toon->nameatrealm);
    $variables['nameatrealmlink'] = l($achievement->toon->nameatrealm, 'toon/' . $achievement->toon->tid);
  }

  $variables['content'] = array();
  // Helpful $content variable for templates.
  foreach (element_children($variables['elements']) as $key) {
    $variables['content'][$key] = $variables['elements'][$key];
  }

  // Make the field variables available with the appropriate language.
  field_attach_preprocess('wowachievement', $achievement, $variables['content'], $variables);

  // Suggest using the view_mode template.
  $variables['theme_hook_suggestions'][] = 'wowachievement_' . $variables['view_mode'];
}